/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package reconhecimento;

import java.io.File;
import java.io.FilenameFilter;
import java.nio.IntBuffer;
import static org.bytedeco.javacpp.opencv_core.CV_32SC1;
import org.bytedeco.javacpp.opencv_core.Mat;
import org.bytedeco.javacpp.opencv_core.MatVector;
import org.bytedeco.javacpp.opencv_core.Size;
import org.bytedeco.javacpp.opencv_face.FaceRecognizer;
import static org.bytedeco.javacpp.opencv_face.createEigenFaceRecognizer;
import static org.bytedeco.javacpp.opencv_face.createFisherFaceRecognizer;
import static org.bytedeco.javacpp.opencv_face.createLBPHFaceRecognizer;
import static org.bytedeco.javacpp.opencv_imgcodecs.CV_LOAD_IMAGE_GRAYSCALE;
import static org.bytedeco.javacpp.opencv_imgcodecs.imread;
import static org.bytedeco.javacpp.opencv_imgproc.resize;

/**
 *
 * @author Leonel Ceolin Farias
 */
public class Treinamento { //código responsável pelo treinamento dos algoritmos para o reconhecimento

    public static void main(String args[]) {
        File diretorio = new File("src\\fotos"); //acessa a pasta de fotos para realizar o treinamento dos algoritmos
        FilenameFilter filtroImagem = new FilenameFilter() { //filtra dentro da pasta as extensões de imagens aceitas
            @Override
            public boolean accept(File dir, String nome) {
                return nome.endsWith(".jpg") || nome.endsWith(".gif") || nome.endsWith(".png") || nome.endsWith(".jpeg");
            }
        };

        File[] arquivos = diretorio.listFiles(filtroImagem); //lista todos os arquivos(imagens)
        MatVector fotos = new MatVector(arquivos.length); //MatVector - vetor de imagens que contém todas as fotos da pasta
        Mat rotulos = new Mat(arquivos.length, 1, CV_32SC1); //cria os rotulos / rotulos são os id atribuidos no caso "pessoa.1 ou pessoa.2"
        IntBuffer rotulosBuffer = rotulos.createBuffer(); // IntBuffer - armazenar de forma correta os rotulos
        int contador = 0; //conta a quantidade de imagens
        
        for (File imagem : arquivos) { //percorre todas as imagens na pasta
            Mat foto = imread(imagem.getAbsolutePath(), CV_LOAD_IMAGE_GRAYSCALE); //converte as imagens para escala de cinza
            int classe = Integer.parseInt(imagem.getName().split("\\.")[1]); //identifica se é pessoa 1 ou 2
            //System.out.println(classe);
            resize(foto, foto, new Size(160, 160)); //redimensiona as fotos para o tamanho 160
            fotos.put(contador, foto);
            rotulosBuffer.put(contador, classe);
            contador++;
        }

        FaceRecognizer eigenfaces = createEigenFaceRecognizer();
        FaceRecognizer fisherfaces = createFisherFaceRecognizer();
        FaceRecognizer lbph = createLBPHFaceRecognizer(2, 9, 9, 9, 1);
        //parametros LBPHF (radius, neighboors,grid_x, gridd_y, threshold)
        eigenfaces.train(fotos, rotulos);
        eigenfaces.save("src\\recursos\\classificadorEigenFaces.yml");
        fisherfaces.train(fotos, rotulos);
        fisherfaces.save("src\\recursos\\classificadorFisherFaces.yml");
        lbph.train(fotos, rotulos);
        lbph.save("src\\recursos\\classificadorLBPH.yml");
    }
}
